#BEGIN COPYRIGHT
#*************************************************************************
#
# Licensed Materials - Property of IBM
# 5725-C94, 5725-C95, 5725-C96
# (C) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
#*************************************************************************
#END COPYRIGHT

import os, re
from java.lang import System
from java.io import FileInputStream
from java.io import BufferedInputStream
from java.util import Properties
from javax.xml.parsers import DocumentBuilderFactory
from org.apache.xml.serialize import XMLSerializer

AdminConfig.setValidationLevel("NONE" )

#------------------------------------------------------------------------------
# Load property file
#------------------------------------------------------------------------------
def loadProperties(source):  
  result = {}  
  if type(source) == type(''):    # name provided, use file  
    source = FileInputStream(source)  
  bis = BufferedInputStream(source)  
  props = Properties()  
  props.load(bis)   
  bis.close()  
  for key in props.keySet().iterator():  
    result[key] = props.get(key)  
  return result
#end of loadProperties()


#------------------------------------------------------------------------------
# Get property value by property name
#------------------------------------------------------------------------------
def getProperty (properties, name, default=None):  
  return properties.get(name, default)    
#end of getProperty()


#------------------------------------------------------------------------------
# Get clusters by DE name
#------------------------------------------------------------------------------
def getClusters(topologyName):  
  topologyClusters = []  
  
  # identify the topology's clusters (in a certain order of preference)
  bpmClusters = AdminUtilities.convertToList(AdminConfig.list('BPMCluster', AdminConfig.getid('/BPMDeploymentEnvironment:%s/' % topologyName)))
  for bpmCluster in bpmClusters:
    clusterName = AdminConfig.showAttribute(bpmCluster, 'name')
    cluster = AdminConfig.getid('/ServerCluster:%s' % clusterName)
    if cluster not in topologyClusters:
      topologyClusters.append(cluster)
        
  return   topologyClusters
#end of getClusters()


#------------------------------------------------------------------------------
# Get cluster capabilities by cluster name
#------------------------------------------------------------------------------
def getClusterCapabilities(topologyName, clusterName):  
  capabilities = []  
  
  bpmClusters = AdminUtilities.convertToList(AdminConfig.list('BPMCluster', AdminConfig.getid('/BPMDeploymentEnvironment:%s/' % topologyName)))
  for bpmCluster in bpmClusters:
    cluster = AdminConfig.showAttribute(bpmCluster, 'name')
    if (cluster == clusterName):
      capabilities = AdminConfig.showAttribute(bpmCluster, 'capabilities')
      break;
        
  return   capabilities
#end of getClusterCapabilities()


#------------------------------------------------------------------------------
# Get cluster members by cluster name
#------------------------------------------------------------------------------
def getClusterMembers(clusterName):  
    clusterMembers = []  
    cluster = AdminConfig.getid('/ServerCluster:%s' % clusterName)
    clusterMembers = AdminUtilities.convertToList(AdminConfig.list('ClusterMember', cluster))
  
    return clusterMembers
#end of getClusterMembers()


#------------------------------------------------------------------------------
# Modify JVM heap size
#------------------------------------------------------------------------------
def modifyJvmHeapSize(server, minHeap, maxHeap, nurHeapMin, nurHeapMax, heapBase):
  # Accessing JVM config
  print "Accessing JVM configuration..."
  jvm = AdminConfig.list("JavaVirtualMachine", server)

  print "Initial Heap Size (old/new):  "+AdminConfig.showAttribute(jvm, "initialHeapSize")+"/"+str(minHeap)
  if (minHeap!=DEFAULT):
    attrs= [["initialHeapSize", minHeap]]
    print "Updating initialHeapSize of jvm..."
    AdminConfig.modify(jvm, attrs)
  else:
    print "Skip initialHeapSize of jvm"

  print "Maximum Heap Size (old/new):  "+AdminConfig.showAttribute(jvm, "maximumHeapSize")+"/"+str(maxHeap)
  if (maxHeap!=DEFAULT):
    attrs = [["maximumHeapSize", maxHeap]]
    print "Updating maxHeap of jvm..."
    AdminConfig.modify(jvm, attrs)
  else:
    print "Skip maxHeap of jvm"
  
  if (nurHeapMin!=DEFAULT):
    oldGenericJvmArguments = AdminConfig.showAttribute(jvm, "genericJvmArguments")
    newGenericJvmArguments = oldGenericJvmArguments+" -Xmns"+str(nurHeapMin)+"m"
    if (oldGenericJvmArguments.find("-Xmns")!=-1):
      newGenericJvmArguments = re.subn("-Xmns\d*[mM]", "-Xmns"+str(nurHeapMin)+"m", oldGenericJvmArguments)[0]
    print "Old generic JVM Args:  "+`oldGenericJvmArguments`;
    print "New generic JVM Args:  "+`newGenericJvmArguments`;
    attrs = [["genericJvmArguments", newGenericJvmArguments]]
    print "Updating generic JVM Args of jvm for Xmns... "
    AdminConfig.modify(jvm, attrs)
  else:
    print "Skip JVM Args of jvm for Xmns"
  
  if (nurHeapMax!=DEFAULT):
    oldGenericJvmArguments = AdminConfig.showAttribute(jvm, "genericJvmArguments")
    newGenericJvmArguments = oldGenericJvmArguments+" -Xmnx"+str(nurHeapMax)+"m"
    if (oldGenericJvmArguments.find("-Xmnx")!=-1):
      newGenericJvmArguments = re.subn("-Xmnx\d*[mM]", "-Xmnx"+str(nurHeapMax)+"m", oldGenericJvmArguments)[0]
    print "Old generic JVM Args:  "+`oldGenericJvmArguments`;
    print "New generic JVM Args:  "+`newGenericJvmArguments`;
    attrs = [["genericJvmArguments", newGenericJvmArguments]]
    print "Updating generic JVM Args of jvm for Xmnx... "
    AdminConfig.modify(jvm, attrs)
  else:
    print "Skip JVM Args of jvm for Xmnx"
  
  if (heapBase!=DEFAULT and 'aix'!=System.getProperty("os.name").lower()):
    oldGenericJvmArguments = AdminConfig.showAttribute(jvm, "genericJvmArguments")
    newGenericJvmArguments = oldGenericJvmArguments
    if (heapBase=='Disable'):
      # disable compressed reference by using -Xnocompressedrefs
      if (oldGenericJvmArguments.find("-Xgc:preferredHeapBase=")!=-1):
        newGenericJvmArguments = re.subn("-Xgc:preferredHeapBase=0x[A-Fa-f\d]*", "-Xnocompressedrefs", oldGenericJvmArguments)[0]
      elif (oldGenericJvmArguments.find("-Xnocompressedrefs")==-1):
        newGenericJvmArguments = oldGenericJvmArguments + " -Xnocompressedrefs"
    # end if Disable
    else:
      # enable compressed reference by removing -Xnocompressedrefs and apply -Xgc:preferredHeapBase
      if (oldGenericJvmArguments.find("-Xnocompressedrefs")!=-1):
        # was disabled, re-enable it
        newGenericJvmArguments = re.subn("-Xnocompressedrefs", "-Xgc:preferredHeapBase="+str(heapBase), oldGenericJvmArguments)[0]
      elif (oldGenericJvmArguments.find("-Xgc:preferredHeapBase=")!=-1):
        # already used preferredHeapBase
        newGenericJvmArguments = re.subn("-Xgc:preferredHeapBase=0x[A-Fa-f\d]*", "-Xgc:preferredHeapBase="+str(heapBase), oldGenericJvmArguments)[0]
      else:
        # not used preferredHeapBase yet
        newGenericJvmArguments = oldGenericJvmArguments+" -Xgc:preferredHeapBase="+str(heapBase)
    # end if enable      
    print "Old generic JVM Args:  "+`oldGenericJvmArguments`;
    print "New generic JVM Args:  "+`newGenericJvmArguments`;
    attrs = [["genericJvmArguments", newGenericJvmArguments]]
    print "Updating generic JVM Args of jvm for Heap Base... "
    AdminConfig.modify(jvm, attrs)
  else:
    print "Skip JVM Args of jvm for Heap Base, OS name: "+System.getProperty("os.name").lower()

  # enable "Verbose Garbage Collection"
  print "Enabling verboseModeGarbageCollection"
  attrs=[["verboseModeGarbageCollection", "true"]]
  AdminConfig.modify(jvm, attrs)  
  print ""
#end of modifyJvmHeapSize()


#------------------------------------------------------------------------------
# Modify thread pool
#------------------------------------------------------------------------------
def modifyThreadPool(defPoolMin, defPoolMax):
  # Accessing web container thread pool config
  if (defPoolMax != DEFAULT or defPoolMin != DEFAULT):
    print "Accessing default thread pool configuration..."
    tpList = AdminConfig.list("ThreadPool", server)
    defaultPool = ""
    for pool in AdminUtilities.convertToList(tpList):
      pool = pool.rstrip()
      if (getName(pool) == "Default"):
        defaultPool = pool
        break
        #endIf
    #endFor
  
    print "Default ThreadPool MinSize (old/new):             "+AdminConfig.showAttribute(defaultPool, "minimumSize")+"/"+str(defPoolMin)
    print "Default ThreadPool MaxSize (old/new):             "+AdminConfig.showAttribute(defaultPool, "maximumSize")+"/"+str(defPoolMax)
  
    if (defPoolMax != DEFAULT):
      attrs = [["maximumSize", defPoolMax]]
      print "Updating maximumSize of default thread pool..."
      AdminConfig.modify(defaultPool, attrs )
    else:
      print "Skip maximumSize of default thread pool"
    
    if (defPoolMin != DEFAULT):
      attrs = [["minimumSize", defPoolMin]]
      AdminConfig.modify(defaultPool, attrs )
      print "Updating minimumSize of default thread pool..."
    else:
      print "Skip minimumSize of default thread pool"
  else:
    print "Skip configuring default thread pool"
  print ""
#end of modifyThreadPool()


#---------------------------------------------
# Misc Procedures
#---------------------------------------------
def getName(objectId):
  endIndex = (objectId.find("(c") - 1)
  stIndex = 0
  if (objectId.find("\"") == 0):
    stIndex = 1
  #endIf
  return objectId[stIndex:endIndex+1]
#end of getName()


#------------------------------------------------------------------------------
# Modify data source
#------------------------------------------------------------------------------
def modifyDataSource(topologyCluster, dsConPool):
  if (dsConPool!=DEFAULT):
    print "Accessing non-ME Datasorce config"
    # Accessing non-ME Datasource config
    dsList = AdminUtilities.convertToList(AdminConfig.list("DataSource", topologyCluster))
    for ds in dsList:
      dsName = AdminConfig.showAttribute(ds, 'name')
      if (dsName.find(" Messaging ")==-1):
        conPool=AdminConfig.showAttribute(ds,'connectionPool')
        print "Max connection pool size of "+str(dsName)+"(old/new):  "+AdminConfig.showAttribute(conPool,"maxConnections")+"/"+str(dsConPool)
        attrs=[["maxConnections",dsConPool]]
        AdminConfig.modify(conPool,attrs)
  else:
    print "Skip non-ME Datasorce config"
  print ""
#end of modifyDataSource()


#------------------------------------------------------------------------------
# Modify activation spec
#------------------------------------------------------------------------------
def modifyActivationSpec(acMaxConcurrency, acMaxBatchSize):
  print "Accessing SCA_WLE Activation Spec config"
  # Accessing SCA_WLE Activation Spec config
  if (acMaxConcurrency!=DEFAULT or acMaxBatchSize!=DEFAULT):
    acList = AdminUtilities.convertToList(AdminConfig.list("J2CActivationSpec"))
    for ac in acList:
      acName = AdminConfig.showAttribute(ac, 'name')
      if (acName.startswith("SCA_WLE")):
        propList=AdminConfig.showAttribute(ac, 'resourceProperties').replace('[','').replace(']','').split(' ')
        for prop in propList:
          propName = AdminConfig.showAttribute(prop, 'name')
          if (propName=='maxConcurrency'):
            print  "maxConcurrency of "+str(acName)+"(old/new):  "+AdminConfig.showAttribute(prop, "value")+"/"+str(acMaxConcurrency)
            if (acMaxConcurrency!=DEFAULT):
              attrs=[["value",acMaxConcurrency]]
              print "Updating maxConcurrency of "+str(acName)
              AdminConfig.modify(prop,attrs)
            else:
              print "Skip maxConcurrency of "+str(acName)  
          elif (propName=='maxBatchSize'):
            print  "maxBatchSize of "+str(acName)+"(old/new):  "+AdminConfig.showAttribute(prop, "value")+"/"+str(acMaxBatchSize)
            if (acMaxBatchSize!=DEFAULT):
              attrs=[["value",acMaxBatchSize]]
              print "Updating maxBatchSize of "+str(acName)
              AdminConfig.modify(prop,attrs)
            else:
              print "Skip maxBatchSize of "+str(acName)  
  else:
    print "Skip configure SCA_WLE Activation Spec config" 
  print ""
#end of modifyActivationSpec()


#------------------------------------------------------------------------------
# Modify event manager
#------------------------------------------------------------------------------
def modifyEventManager(clusterName, environmentType, bpdCapacity, threadPoolSize, applyWLESettingToCluster, sendExternalEmail):
  # Accessing WLE specific settings (such as bpd-queue-capacity and max-thread-pool-size that can be find in <profile>/config/cells/<cell name>nodes/<node name>/servers/server1/process-center/config/system/80EventManager.xml)
  if (bpdCapacity==DEFAULT and threadPoolSize==DEFAULT):
    print "Skip updating bpd-queue-capacity and max-thread-pool-size"
  else:
    cell=AdminConfig.list('Cell')
    cellName=AdminConfig.showAttribute(cell,'name')
    #node=AdminConfig.list('Node')
    #nodeName=AdminConfig.showAttribute(node,'name')
    pathPrefix='cells'+SEPARATOR+str(cellName)+SEPARATOR+'nodes'+SEPARATOR+str(nodeName)+SEPARATOR+'servers'+SEPARATOR+serverName
    modifyWLESettings(pathPrefix, bpdCapacity, threadPoolSize)
    if (applyWLESettingToCluster != DEFAULT and applyWLESettingToCluster != 'false' and applyWLESettingToCluster != 'no'):
      pathPrefix='cells'+SEPARATOR+str(cellName)+SEPARATOR+'clusters'+SEPARATOR+clusterName
      modifyWLESettings(pathPrefix, bpdCapacity, threadPoolSize)
      #end if
    #end else

  # Accessing 99Local.xml to set send-external-email to false
  if(sendExternalEmail==DEFAULT):
    print "Skip updating send-external-email"
  else:
    cell=AdminConfig.list('Cell')
    cellName=AdminConfig.showAttribute(cell,'name')
    
    #in ND cluster
    if(environmentType=="Process Center"):
      server_Type="process-center"
    else:
      server_Type="process-server"
    pathPrefix='cells'+SEPARATOR+str(cellName)+SEPARATOR+'clusters'+SEPARATOR+clusterName+SEPARATOR+str(server_Type)+SEPARATOR+'config'+SEPARATOR+'system'
    disableExternalMailSetting(pathPrefix, sendExternalEmail)
    #in ND cluster members
    pathPrefix='cells'+SEPARATOR+str(cellName)+SEPARATOR+'nodes'+SEPARATOR+str(nodeName)+SEPARATOR+'servers'+SEPARATOR+serverName+SEPARATOR+str(server_Type)+SEPARATOR+'config'+SEPARATOR+'system'
    disableExternalMailSetting(pathPrefix, sendExternalEmail)
  print ""
#end of modifyEventManager()


#------------------------------------------------------------------------------
# Modify WLE settings in 80EventManager.xml
#------------------------------------------------------------------------------
def modifyWLESettings(emFilePathPrefix, bpdCapacity, threadPoolSize):
  global emFilePath,obj
  emFilePath=emFilePathPrefix
  tmpFilePath=profileHome+SEPARATOR+'temp'+SEPARATOR+'80EventManager.xml'
  obj=None
  try:
    emFilePath+=SEPARATOR+'process-server'+SEPARATOR+'config'+SEPARATOR+'system'+SEPARATOR+'80EventManager.xml'
    obj=AdminConfig.extract(emFilePath, tmpFilePath)
  except:
    emFilePath= emFilePathPrefix + SEPARATOR+'process-center'+SEPARATOR+'config'+SEPARATOR+'system'+SEPARATOR+'80EventManager.xml'
    obj=AdminConfig.extract(emFilePath, tmpFilePath)
    print "file "+emFilePath+" extracted"
  # xml.dom.minidom has bug and doesn't work, so have to use java's parser
  print "global path:"+ emFilePath
  factory = DocumentBuilderFactory.newInstance()
  builder = factory.newDocumentBuilder()
  input = FileInputStream(tmpFilePath)
  doc = builder.parse(input)
  nodeList = doc.getElementsByTagName("bpd-queue-capacity")
  eleBpd=nodeList.item(0)
  oldBpd=eleBpd.getTextContent()
  print "bpd-queue-capacity of 80EventManager.xml(old/new):  "+str(oldBpd)+"/"+str(bpdCapacity)
  if (bpdCapacity!=DEFAULT):
    print "Updating bpd-queue-capacity"
    eleBpd.setTextContent(str(bpdCapacity))
  else:
    print "Skip bpd-queue-capacity"
    
  nodeList = doc.getElementsByTagName("max-thread-pool-size")
  elePoolSize=nodeList.item(0)
  oldPoolSize=elePoolSize.getTextContent()
  print "max-thread-pool-size of 80EventManager.xml(old/new):  "+str(oldPoolSize)+"/"+str(threadPoolSize)
  if (threadPoolSize!=DEFAULT):
    print "Updating max-thread-pool-size"
    elePoolSize.setTextContent(str(threadPoolSize))
  else:
    print "Skip max-thread-pool-size"
    
  output=open(tmpFilePath,'w')
  serializer = XMLSerializer()
  serializer.setOutputByteStream(output)
  serializer.serialize(doc)

  AdminConfig.checkin(emFilePath, tmpFilePath,obj)
  print "80EventManager.xml updated"
  #os.remove(tmpFilePath)
# end of modifyWLESettings()


#------------------------------------------------------------------------------
# Disable send-external-mail in 99Local.xml
#------------------------------------------------------------------------------
def disableExternalMailSetting(emFilePathPrefix, sendExternalEmail):
  global emFilePath,obj
  emFilePath=emFilePathPrefix
  tmpFilePath=profileHome+SEPARATOR+'temp'+SEPARATOR+'99Local.xml'
  obj=None
  try:
    emFilePath+=SEPARATOR+'99Local.xml'
    obj=AdminConfig.extract(emFilePath, tmpFilePath)
  except:
    emFilePath= emFilePathPrefix + SEPARATOR+'99Local.xml'
    obj=AdminConfig.extract(emFilePath, tmpFilePath)
    print "file "+emFilePath+" extracted"
  # xml.dom.minidom has bug and doesn't work, so have to use java's parser
  print "global path:"+ emFilePath
  factory = DocumentBuilderFactory.newInstance()
  builder = factory.newDocumentBuilder()
  input = FileInputStream(tmpFilePath)
  doc = builder.parse(input)
  nodeList = doc.getElementsByTagName("send-external-email")
  eleBpd=nodeList.item(0)
  oldBpd=eleBpd.getTextContent()
  print "send-external-email of 99Local.xml(old/new):  "+str(oldBpd)+"/"+str(sendExternalEmail)
  eleBpd.setTextContent("false")
    
  output=open(tmpFilePath,'w')
  serializer = XMLSerializer()
  serializer.setOutputByteStream(output)
  serializer.serialize(doc)

  AdminConfig.checkin(emFilePath, tmpFilePath,obj)
  print "99Local.xml updated"
  #os.remove(tmpFilePath)
# end of disableExternalMailSetting()


#------------------------------------------------------------------------------
# Modify connection factory
#------------------------------------------------------------------------------
def modifyConnectionFactory(twClientCFMax, cacheMessCFMax):
  # Accessing Connection Factory Properties
  cfList = AdminUtilities.convertToList(AdminConfig.list("J2CConnectionFactory"))
  for cf in cfList:
    if (cf == ''):
      continue
    cfName = AdminConfig.showAttribute(cf, 'name')
    if (cfName=='TWClientConnectionFactory'):
      if (twClientCFMax!=DEFAULT):
        cfPool= AdminConfig.showAttribute(cf, 'connectionPool')
        attrs=[["maxConnections",twClientCFMax]]
        print "Updating maxConnections of "+str(cfName)+"(old/new):  "+AdminConfig.showAttribute(cfPool, "maxConnections")+"/"+str(twClientCFMax)
        AdminConfig.modify(cfPool,attrs)
      else:
        print "Skip maxConnections of "+str(cfName) 
    #end if TWClientConnectionFactory
   
    if (cfName=='cacheMessageConnectionFactory'):
      if (cacheMessCFMax!=DEFAULT):
        cfPool= AdminConfig.showAttribute(cf, 'connectionPool')
        attrs=[["maxConnections",cacheMessCFMax]]
        print "Updating maxConnections of "+str(cfName)+"(old/new):  "+AdminConfig.showAttribute(cfPool, "maxConnections")+"/"+str(cacheMessCFMax)
        AdminConfig.modify(cfPool,attrs)
      else:
        print "Skip maxConnections of "+str(cfName) 
    #end if cacheMessageConnectionFactory
  print ""
#end of modifyConnectionFactory()

#------------------------------------------------------------------------------
# Modify ORB service
#------------------------------------------------------------------------------
def modifyORBService(corbaFragSize):
  # Accessing ORB Service Properties
  # there should be only one ORB entry in a standalone server
  if (corbaFragSize!=DEFAULT):
    ORB = AdminConfig.list("ObjectRequestBroker")
    ORBProperties = AdminUtilities.convertToList(AdminConfig.list("Property", ORB))
    corbaFragSizeExists = 0
    # check if the property already exists
    for ORBProp in ORBProperties:
      ORBPropName = AdminConfig.showAttribute(ORBProp, 'name')
      if (ORBPropName=='com.ibm.CORBA.FragmentSize'):
        print "The com.ibm.CORBA.FragmentSize property already exists."
        oldCorbaFragSize=AdminConfig.showAttribute(ORBProp, 'value')
        attrs=[["value",corbaFragSize]]
        print "Updating com.ibm.CORBA.FragmentSize of ORB Service (old/new):  "+AdminConfig.showAttribute(ORBProp, 'value')+"/"+str(corbaFragSize)
        AdminConfig.modify(ORBProp,attrs)
        corbaFragSizeExists=1
        break
      # end if ORBPropName
    #end for ORBProp
    
    if (not corbaFragSizeExists):
      attrs=[["name","com.ibm.CORBA.FragmentSize"], ["value", corbaFragSize]]
      print "Creating new property com.ibm.CORBA.FragmentSize="+str(corbaFragSize)+" for ORB Service"
      AdminConfig.create("Property",ORB, attrs)
    #end if not corbaFragSizeExists
  #end if corbaFragSize
  else:
    print "Skip updating com.ibm.CORBA.FragmentSize of ORB Service"  
  print ""
#end of modifyORBService()


#------------------------------------------------------------------------------
# Main
#------------------------------------------------------------------------------
print ""
print "Starting script..."
print ""

DEFAULT='Default'
SEPARATOR=os.sep

print "Command line arguments:"
if (len(sys.argv) < 2):
  sys.exit()
else:
  #parse the parameters from command line
  deName=sys.argv[0]
  propertyFile=sys.argv[1]
  profileHome=System.getProperty('user.install.root')
  
  print "deName: "+deName
  print "propertyFile: "+propertyFile
  print "profileHome: "+profileHome
  print ""

  print "Reading configuration parameters from property file..."
  print ""
  
  props = loadProperties(propertyFile)
  
  dmgr_HEAP_MIN=getProperty(props, 'dmgr.minHeap', '768')
  dmgr_HEAP_MAX=getProperty(props, 'dmgr.maxHeap', '2048')

  node_HEAP_MIN=getProperty(props, 'node.minHeap', '128')
  node_HEAP_MAX=getProperty(props, 'node.maxHeap', '768')

  appCluster_HEAP_MIN=getProperty(props, 'appCluster.minHeap', '1536')
  appCluster_HEAP_MAX=getProperty(props, 'appCluster.maxHeap', '2560')
  appCluster_HEAP_NUR_MIN=getProperty(props, 'appCluster.nurHeapMin', '256')
  appCluster_HEAP_NUR_MAX=getProperty(props, 'appCluster.nurHeapMax', '768')
  appCluster_HEAP_BASE=getProperty(props, 'appCluster.heapBase', '0x100000000')
  appCluster_DEF_POOL_MIN=getProperty(props, 'appCluster.defPoolMin', '20')
  appCluster_DEF_POOL_MAX=getProperty(props, 'appCluster.defPoolMax', '40')
  appCluster_DS_CON_POOL=getProperty(props, 'appCluster.dsConPool', '100')
  appCluster_AC_MAX_CONCURRENCY=getProperty(props, 'appCluster.scaWleAcMaxConcurrency', '20')  
  appCluster_AC_MAX_BATCH_SIZE=getProperty(props, 'appCluster.scaWleAcMaxBatchSize', '8')
  appCluster_BPD_CAPACITY=getProperty(props, 'appCluster.bpdCapacity', '80')
  appCluster_THREAD_POOL_SIZE=getProperty(props, 'appCluster.threadPoolSize', '110')
  appCluster_SEND_EXTERNAL_EMAIL=getProperty(props, 'appCluster.sendExternalEmail', 'false')
  appCluster_APPLY_WLE_SETTING_TO_CLUSTER=getProperty(props, 'appCluster.applyWLESettingToCluster', 'true')
  appCluster_TW_CLIENT_CF_MAX=getProperty(props, 'appCluster.twClientCFMax', '60')
  appCluster_CACHE_MESS_CF_MAX=getProperty(props, 'appCluster.cacheMessCFMax', '60')

  meCluster_HEAP_MIN=getProperty(props, 'meCluster.minHeap', '256')
  meCluster_HEAP_MAX=getProperty(props, 'meCluster.maxHeap', '1024')

  supCluster_HEAP_MIN=getProperty(props, 'supCluster.minHeap', '256')
  supCluster_HEAP_MAX=getProperty(props, 'supCluster.maxHeap', '1024')
    #endElse

print "Dmgr configuration..."
print "--------------------------"
dmgr=AdminConfig.getid('/Cell:/Node:/Server:dmgr/')
modifyJvmHeapSize(dmgr, dmgr_HEAP_MIN, dmgr_HEAP_MAX, DEFAULT, DEFAULT, DEFAULT)
print "Dmgr configuration completed..."
print ""

bpmDe = AdminConfig.getid('/BPMDeploymentEnvironment:%s/' % deName)
environmentType = AdminConfig.showAttribute(bpmDe, 'environmentType')
print "Deployment environment type is: "+environmentType
print ""

topologyClusters = getClusters(deName)
tunedNodes = []
for topologyCluster in topologyClusters:
  clusterName = AdminConfig.showAttribute(topologyCluster, 'name')
  clusterMembers = getClusterMembers(clusterName)
  for clusterMember in clusterMembers:  
    serverName = AdminConfig.showAttribute(clusterMember, 'memberName')
    nodeName = AdminConfig.showAttribute(clusterMember, 'nodeName')
    if nodeName not in tunedNodes:
      print "Custom node "+nodeName+" configuratoin..."
      print "------------------------------------------------"
      nodeAgent=AdminConfig.getid('/Cell:/Node:%s/Server:nodeagent/' % (nodeName))
      modifyJvmHeapSize(nodeAgent, node_HEAP_MIN, node_HEAP_MAX, DEFAULT, DEFAULT, DEFAULT)
      tunedNodes.append(nodeName)
      print "Custom node configuration completed..."
      print ""
    server = AdminConfig.getid('/Cell:/Node:%s/Server:%s/' % (nodeName, serverName))
    # standalone profile only
    #server = AdminConfig.list('Server')
    print "Cluster member "+serverName+" configuration..."
    print "-----------------------------------------------------"
    clusterCapabilities = getClusterCapabilities(deName, clusterName).split(';')
    if clusterCapabilities and 'Messaging' in clusterCapabilities:
      modifyJvmHeapSize(server, meCluster_HEAP_MIN, meCluster_HEAP_MAX, DEFAULT, DEFAULT, DEFAULT)
    if clusterCapabilities and 'Support' in clusterCapabilities:
      modifyJvmHeapSize(server, supCluster_HEAP_MIN, supCluster_HEAP_MAX, DEFAULT, DEFAULT, DEFAULT)
    if clusterCapabilities and 'Application' in clusterCapabilities:
      modifyJvmHeapSize(server, appCluster_HEAP_MIN, appCluster_HEAP_MAX, appCluster_HEAP_NUR_MIN, appCluster_HEAP_NUR_MAX, appCluster_HEAP_BASE)
      modifyThreadPool(appCluster_DEF_POOL_MIN, appCluster_DEF_POOL_MAX)
      modifyDataSource(topologyCluster, appCluster_DS_CON_POOL)
      if (environmentType == 'Process Server'):
        modifyActivationSpec(appCluster_AC_MAX_CONCURRENCY, appCluster_AC_MAX_BATCH_SIZE)
        modifyEventManager(clusterName, environmentType, appCluster_BPD_CAPACITY, appCluster_THREAD_POOL_SIZE, appCluster_APPLY_WLE_SETTING_TO_CLUSTER, appCluster_SEND_EXTERNAL_EMAIL)
      else:
        modifyEventManager(clusterName, environmentType, DEFAULT, DEFAULT, DEFAULT, appCluster_SEND_EXTERNAL_EMAIL)
        modifyConnectionFactory(appCluster_TW_CLIENT_CF_MAX, appCluster_CACHE_MESS_CF_MAX)
    print "Cluster member "+serverName+" configuration completed..."
    print ""

print ""
print "Script completed..."
print "Saving configuration..."

# commit all the changes above
AdminConfig.save()
